from Products.PloneTestCase import ptc
from Testing import ZopeTestCase
from ftw.blog.tests import layer
import doctest
import unittest


MODULENAMES = ()


TESTFILES = (
    'blog.txt',
    )


OPTIONFLAGS = (doctest.NORMALIZE_WHITESPACE |
               doctest.ELLIPSIS |
               doctest.REPORT_NDIFF)


def test_suite():

    suite = unittest.TestSuite()

    for testfile in TESTFILES:
        fdfs = ZopeTestCase.FunctionalDocFileSuite(
            testfile,
            optionflags=OPTIONFLAGS,
            test_class=ptc.FunctionalTestCase,)
        fdfs.layer = layer.layer
        suite.addTest(fdfs)

    for module in MODULENAMES:
        fdts = ZopeTestCase.FunctionalDocTestSuite(
            module,
            optionflags=OPTIONFLAGS,
            test_class=ptc.FunctionalTestCase)
        fdts.layer = layer.layer
        suite.addTest(fdts)

    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



"""Definition of the Blog Category content type
"""
from zope.interface import implements
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import schemata

from Products.ATContentTypes.config import HAS_LINGUA_PLONE
if HAS_LINGUA_PLONE:
    from Products.LinguaPlone.public import registerType
else:
    from Products.Archetypes.atapi import registerType

from ftw.blog.interfaces import IBlogCategory
from ftw.blog.config import PROJECTNAME


BlogCategorySchema = folder.ATFolderSchema.copy()
schemata.finalizeATCTSchema(BlogCategorySchema,
                            folderish=True,
                            moveDiscussion=False)


class BlogCategory(folder.ATFolder):
    """Blog Category"""
    implements(IBlogCategory)

    portal_type = "BlogCategory"
    schema = BlogCategorySchema

registerType(BlogCategory, PROJECTNAME)



from ftw.tabbedview.browser.tabbed import TabbedView
from ftw.table import helper
from ftw.tabbedview.browser.listing import CatalogListingView
from ftw.blog import _


class BlogsView(TabbedView):
    """Tabbed blog overview"""

    def get_tabs(self):
        return [{'id':'blogs', 'class':''},
                {'id':'blogentries', 'class':''}, ]




class Tab(CatalogListingView):
    """Search for the hole plone site"""

    def update_config(self):
        super(Tab, self).update_config()
        self.filter_path = '/'.join(
            self.context.portal_url.getPortalObject().getPhysicalPath())

class BlogsTab(Tab):

    types = 'Blog'
    sort_on = 'sortable_title'
    show_selects = False
    show_menu = False

    columns = (
        {'column': 'Title',
         'column_title': _(u'column_title', default=u'Title'),
         'sort_index': 'sortable_title',
         'transform': helper.linked},
         {'column': 'Creator',
          'column_title': _(u'column_creator', default=u'Creator'),
          'sort_index': 'sortable_creator',
          'transform': helper.readable_author}, )


class BlogEntriesTab(Tab):

    types = 'BlogEntry'
    sort_on = 'created'
    sort_reverse = True
    show_selects = False
    show_menu = False

    columns = (
        {'column': 'created',
         'column_title': _(u'column_created', default=u'created'),
         'transform': helper.readable_date},
        {'column': 'Title',
         'column_title': _(u'column_title', default=u'Title'),
         'sort_index': 'sortable_title',
         'transform': helper.linked},
         {'column': 'Creator',
          'column_title': _(u'column_creator', default=u'Creator'),
          'sort_index': 'sortable_creator',
          'transform': helper.readable_author}, )



from Products.CMFCore.utils import getToolByName
from plone.app.upgrade.utils import loadMigrationProfile
from zope.annotation.interfaces import IAnnotations
import logging


logger = logging.getLogger('ftw.blog')


def upgrade_to_v2(context):
    """Updates profile
    """
    ppcas = 'plone.portlets.contextassignments'
    catalog = getToolByName(context, 'portal_catalog')
    count = 0
    for brain in catalog(portal_type=['BlogEntry', 'BlogCategory', 'Blog']):
        obj = brain.getObject()
        annotations = IAnnotations(obj)
        if ppcas in annotations.keys():
            if 'blog.portlets' in annotations[ppcas]:
                count += 1
                if len(annotations[ppcas]) == 1:
                    del annotations[ppcas]
                else:
                    del annotations[ppcas]['blog.portlets']

    logger.info("%s %s annotations removed" % (str(count), ppcas))
    loadMigrationProfile(context, 'profile-ftw.blog:to_v2')

    # remove blog_settings actions
    portal_actions = getToolByName(context, 'portal_actions')

    category = portal_actions.get('object_buttons', None)
    action_id = 'blog_settings'
    if category and action_id in category:
        del category[action_id]



"""Common configuration constants
"""

PROJECTNAME = 'ftw.blog'

ADD_PERMISSIONS = {
    # -*- extra stuff goes here -*-
    'BlogEntry': 'ftw.blog: Add BlogEntry',
    'BlogCategory': 'ftw.blog: Add BlogCategory',
    'Blog': 'ftw.blog: Add Blog',
    }

INDEXES = (
    ("getCategoryUids", "KeywordIndex"),
    )

TINYMCE_ALLOWED_BUTTONS = (
    'bg-basicmarkup',
    'bold-button',
    'italic-button',
    'list-ol-addbutton',
    'list-ul-addbutton',
    'definitionlist',
    'linklibdrawer-button',
    'removelink-button',
    'tabledrawer-button',
    )

product_globals = globals()



from Acquisition import aq_inner, aq_parent
from DateTime import DateTime
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import base_hasattr
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.blog.interfaces import IBlogView
from plone.app.content.batching import Batch
from urllib import quote_plus
from zope.i18n import translate
from zope.interface import implements


class BlogView(BrowserView):
    implements(IBlogView)
    """ Shows a Listing of all Blog entries, with the corresponding portlets."""

    template=ViewPageTemplateFile("blog.pt")

    batching=ViewPageTemplateFile("batching.pt")

    filters = []

    def __call__(self):
        """ Get all the Blogentries and return the listingview template.

        It check if the request has some filtering parameters:
        -archiv
        -searchable_text
        -getCategoryUids
        -tag
        """

        context = aq_inner(self.context)
        query = {}
        self.filters = []
        catalog = getToolByName(context, 'portal_catalog')

        if self.request.form.get('archiv'):
            datestr = self.request.form.get('archiv')
            try:
                start = DateTime(datestr)
            except DateTime.SyntaxError:
                start = DateTime(DateTime().strftime("%Y/%m/01"))
            end = DateTime('%s/%s/%s' % (start.year() + start.month() / 12,
                                         start.month() % 12 + 1, 1))
            end = end - 1
            query['created'] = {'query': (start.earliestTime(), end.latestTime()), 'range': 'minmax'}
            month_msgid = 'month_%s' % start.strftime("%b").lower()
            month = translate(month_msgid, domain='plonelocales', context=self.request)
            self.filters.append("%s %s" % (month, start.strftime('%Y')))
        if self.request.form.get('getCategoryUids'):
            uid = self.request.form.get('getCategoryUids')
            category = catalog(UID=uid)[0]
            category_title = category.Title
            if category:
                category_obj = category.getObject()
                if base_hasattr(category_obj, 'getTranslations'):
                    uid = [c.UID() for c in category_obj.getTranslations(review_state=False).values()]
                    translated = category_obj.getTranslation()
                    # If there are no translations, getTranslation returns 'None'
                    if translated:
                        category_title = translated.Title()
            query['getCategoryUids'] = uid
            category = catalog(UID=uid)[0]
            self.filters.append(category_title)
        if self.request.form.get('searchable_text'):
            query['SearchableText'] = self.request.get('searchable_text')
            self.filters.append(query['SearchableText'])
        if self.request.form.get('tag'):
            query['tags'] = self.request.form.get('tag').decode('utf-8')
            self.filters.append(query['tags'])
        if context.portal_type != 'Blog':
            querystring = self.query_string()
            if querystring:
                querystring = '?%s' % querystring
            return self.request.response.redirect(
                aq_parent(context).absolute_url()+ querystring)

        query['sort_on'] = 'created'
        query['sort_order'] = 'reverse'
        query['portal_type'] = 'BlogEntry'
        # show all entries from all languages
        # XXX make this configurable
        if not base_hasattr(context, 'getTranslations'):
            self.entries = context.getFolderContents(contentFilter=query)
        else:
            translations = context.getTranslations().values()
            paths = ['/'.join(tr[0].getPhysicalPath()) for tr in translations]
            query['path'] = paths
            query['Language'] = 'all'
            self.entries = catalog(query)

        pagesize = int(self.request.form.get('pagesize', 5))
        #req.set('pagesize', pagesize)
        pagenumber = int(self.request.form.get('pagenumber', 1))
        #req.set('pagenumber', pagenumber)

        self.batch = Batch(self.entries,
                           pagesize=pagesize, pagenumber=pagenumber, navlistsize=1)

        return self.template()


    def query_string(self, **args):
        """Updates the query string of the current request with the given
        keyword arguments and returns it as a quoted string.
        """
        query = self.request.form.copy()
        # Remove empty query parameters
        for k, v in query.items():
            if v == '':
                del query[k]
        query.update(args)
        return '&'.join(["%s=%s" % (quote_plus(str(k)), quote_plus(str(v)))
                         for k, v in query.items()])



from Acquisition import aq_inner
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.blog.interfaces import IBlogView, IBlogUtils
from zope.component import getUtility
from zope.interface import implements


class RSSView(BrowserView):
    implements(IBlogView)
    """ Shows a Listing of all Blog entries or
    a RSS listing.

    """

    template=ViewPageTemplateFile("rss.pt")

    def __call__(self):
        """ Get all the Blogentries

        and  return the standard rss template.

        """
        context = aq_inner(self.context).aq_explicit
        blogutils = getUtility(IBlogUtils, name='ftw.blog.utils')

        if context.portal_type != 'Blog':
            querystring = context.REQUEST.get('QUERY_STRING', '')
            querystring = querystring and '?' + querystring or querystring
            level = blogutils.getBlogRoot(context)
            url = level.absolute_url() + querystring
            if not self.__name__ == 'rss_blog_view':
                return self.context.REQUEST.RESPONSE.redirect(url)

        self.entries = self.context.getFolderContents({
                'sort_on': 'created',
                'sort_order': 'reverse',
                'portal_type': 'BlogEntry'})

        return self.template(self)



from Acquisition import aq_inner
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.browser.interfaces import ISitemapView
from Products.CMFPlone.browser.navtree import SitemapNavtreeStrategy
from Products.CMFPlone.utils import base_hasattr
from Products.Five import BrowserView
from ftw.blog.interfaces import ICategoryWidget, IBlogUtils
from plone.app.layout.navigation.interfaces import INavtreeStrategy
from plone.app.layout.navigation.navtree import buildFolderTree
from zope.component import getMultiAdapter, getUtility
from zope.interface import implements


class SitemapView(BrowserView):
    """ Return a list of all Categories, with the number of blog entries. """

    implements(ISitemapView)

    def createSiteMap(self):
        context = aq_inner(self.context)
        request = context.REQUEST

        fieldName = request.get('fieldName', False)
        uids = request.get('uids', False)

        view = getMultiAdapter((context, self.request),
                               name='category_widget_builder_view')
        data = view.CategoryMap()

        if data is None:
            return None

        # TODO: The recursion should probably be done in python code
        return context.category_widget_edit_view(
            children=data.get('children', []),
            level=0, bottomLevel=0, uids=uids, fieldName=fieldName)


class CategoryWidgetStrategy(SitemapNavtreeStrategy):
    """Use default sitemap strategy
    """

    def decoratorFactory(self, node):
        ct = getToolByName(self.context, 'portal_catalog')
        oldnode = super(CategoryWidgetStrategy, self).decoratorFactory(node)
        oldnode['uid'] = node['item'].UID

        # Determine the number of blog entries for each category.
        # Also take translations of categories into account.
        # TODO: This will perform badly with a large number of categories
        # We could optimize this by always storing a reference to the
        # canonical category instead of the corresponding translation.
        obj = node['item'].getObject()
        count = 0
        if base_hasattr(obj, 'getTranslations'):
            for translation in obj.getTranslations(review_state=False).values():
                count += len(ct(getCategoryUids=translation.UID(),
                                portal_type='BlogEntry',
                                Language='all',))
        else:
            count = len(ct(getCategoryUids=obj.UID(),
                           portal_type='BlogEntry',
                           Language='all',))

        oldnode['count_refs'] = count
        return oldnode


class SiteMapStructure(BrowserView):
    """ Return a FolderTree from the Categories Folder. """

    implements(ICategoryWidget)

    def CategoryMap(self):
        context = aq_inner(self.context)

        query = {}
        query['portal_type'] = ['BlogCategory']

        strategy = getMultiAdapter((context, self), INavtreeStrategy)

        #some modifications for ftw.blog
        strategy.showAllParents = True
        strategy.excludedIds = {}

        blogutils = getUtility(IBlogUtils, name='ftw.blog.utils')
        bloglevel = blogutils.getBlogRoot(context)

        if bloglevel is None:
            return None

        strategy.rootPath = '/'.join(bloglevel.getPhysicalPath()) + \
            '/categories'

        return buildFolderTree(context,
                               obj=context,
                               query=query,
                               strategy=strategy)



from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import _createObjectByType
from Products.CMFPlone.utils import safe_hasattr
from ftw.blog import _


def objectAddedHandler(obj, event):
    """Handle IObjectAddedEvent
    """

    #adding a categories root object
    if not safe_hasattr(obj.aq_explicit, 'categories', False):
        _createObjectByType('BlogCategory', obj, 'categories')
        category = getattr(obj.aq_explicit, 'categories', False)
        if category:
            category.setTitle(obj.translate(_('Categories')))
            category.reindexObject()


def set_description(obj, event):
    """Get the first 200 Chars from text and set them as descrption
    Use portal_transforms to create plain-text
    """

    portal_transforms = getToolByName(obj, 'portal_transforms')
    ploneview = obj.restrictedTraverse('@@plone')
    text = obj.getField('text').get(obj)
    datastream = portal_transforms.convertTo(
        'text/plain',
        text,
        mimetype='text/html')
    plain_text = datastream.getData()
    # truncate
    length = 200
    truncated = ploneview.cropText(plain_text, length)
    obj.getField('description').set(obj, truncated)
    obj.reindexObject()



"""Definition of the Blog content type
"""
from zope.interface import implements
from AccessControl import ClassSecurityInfo
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import schemata

from Products.ATContentTypes.config import HAS_LINGUA_PLONE
if HAS_LINGUA_PLONE:
    from Products.LinguaPlone.public import registerType
else:
    from Products.Archetypes.atapi import registerType

from ftw.blog.interfaces import IBlog
from ftw.blog.config import PROJECTNAME


BlogSchema = folder.ATFolderSchema.copy()
schemata.finalizeATCTSchema(BlogSchema, folderish=True, moveDiscussion=False)


class Blog(folder.ATFolder):
    """The Ftw Blog Type"""
    implements(IBlog)

    portal_type = "Blog"
    schema = BlogSchema

    security = ClassSecurityInfo()

registerType(Blog, PROJECTNAME)



from Acquisition import aq_inner, aq_parent
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from ftw.blog.interfaces import IBlog, IBlogUtils
from zope.interface import implements


class BlogUtils(object):
    """
    blog utilities
    """
    implements(IBlogUtils)

    def getBlogRoot(self, context):
        obj = context

        while not IPloneSiteRoot.providedBy(obj):
            if IBlog.providedBy(obj):
                return obj
            else:
                obj = aq_parent(aq_inner(obj))

        return None



from DateTime import DateTime
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import base_hasattr
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.blog.interfaces import IBlogUtils
from plone.app.portlets.portlets import base
from plone.memoize.view import memoize
from plone.portlets.interfaces import IPortletDataProvider
from zope.component import getUtility
from zope.i18n import translate
from zope.interface import implements


class IArchivePortlet(IPortletDataProvider):
    """
    """


class Assignment(base.Assignment):
    implements(IArchivePortlet)

    @property
    def title(self):
        return "Blog Archive Portlet"


class Renderer(base.Renderer):

    def __init__(self, context, request, view, manager, data):
        self.context = context
        self.data = data
        self.request = request

    @property
    def available(self):
        """Only show the portlet, when the blog isn't empty
        """
        if self.archive_summary():
            return True
        else:
            return False

    def zLocalizedTime(self, time, long_format=False):
        """Convert time to localized time
        """
        month_msgid = 'month_%s' % time.strftime("%b").lower()
        month = translate(month_msgid, domain='plonelocales', context=self.request)

        return u"%s %s" % (month, time.strftime('%Y'))

    @memoize
    def archive_summary(self):
        """Returns an ordered list of summary infos per month."""
        catalog = getToolByName(self.context, 'portal_catalog')
        query = {}
        blogutils = getUtility(IBlogUtils, name='ftw.blog.utils')
        blogroot = blogutils.getBlogRoot(self.context)
        if base_hasattr(blogroot, 'getTranslations'):
            blogroots = blogroot.getTranslations(review_state=False).values()
            root_path = ['/'.join(br.getPhysicalPath()) for br in blogroots]
            query['Language'] = 'all'
        else:
            root_path ='/'.join(blogroot.getPhysicalPath())

        query['path'] = root_path
        query['portal_type'] = 'BlogEntry'

        archive_counts = {}
        blog_entries = catalog(**query)
        for entry in blog_entries:
            year_month = entry.created.strftime('%Y/%m')
            if year_month in archive_counts:
                archive_counts[year_month] += 1
            else:
                archive_counts[year_month] = 1

        archive_summary = []
        ac_keys = archive_counts.keys()
        ac_keys.sort(reverse=True)
        for year_month in ac_keys:
            archive_summary.append(dict(
                title=self.zLocalizedTime(DateTime('%s/01' % year_month)),
                number=archive_counts[year_month],
                url='%s?archiv=%s/01' % (blogroot.absolute_url(), year_month),
            ))
        return archive_summary

    render = ViewPageTemplateFile('archiv.pt')


class AddForm(base.NullAddForm):

    def create(self):
        return Assignment()



"""Definition of the Blog Entry content type
"""
from zope.interface import implements
from Acquisition import aq_inner
from DateTime import DateTime
from Products.Archetypes import atapi
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import schemata
from Products.ATReferenceBrowserWidget import ATReferenceBrowserWidget

from Products.ATContentTypes.config import HAS_LINGUA_PLONE
if HAS_LINGUA_PLONE:
    from Products.LinguaPlone.public import registerType
else:
    from Products.Archetypes.atapi import registerType

from ftw.blog.interfaces import IBlogEntry
from ftw.blog.config import PROJECTNAME, TINYMCE_ALLOWED_BUTTONS
from ftw.blog import _

BlogEntrySchema = folder.ATFolderSchema.copy() + atapi.Schema((

    atapi.TextField(
        name='text',
        storage=atapi.AnnotationStorage(),
        schemata='default',
        required=False,
        searchable=True,
        default_input_type='text/html',
        default_output_type='text/html',
        widget=atapi.RichWidget(
            label='Text',
            label_msgid='ftw_label_text',
            description='',
            description_msgid='ftw_help_text',
            i18n_domain='ftw.tagging',
            rows=15,
            allow_buttons=TINYMCE_ALLOWED_BUTTONS,
            rooted=True,
        ),
    ),

    atapi.ReferenceField(
        name='categories',
        required=False,
        widget=ATReferenceBrowserWidget.ReferenceBrowserWidget(
            label=_('Categories'),
            allow_browse=False,
            show_results_without_query=True,
            restrict_browsing_to_startup_directory=True,
            base_query={"portal_type": "BlogCategory",
                        "sort_on": "sortable_title"},
            macro='category_reference_widget',
        ),
        multiValued=1,
        schemata='default',
        relationship='blog_categories',
        allowed_types=['BlogCategory'],
    ),

))

# Set storage on fields copied from ATFolderSchema, making sure
# they work well with the python bridge properties.

schemata.finalizeATCTSchema(BlogEntrySchema,
                            folderish=True,
                            moveDiscussion=False)
#finalizeZugSchema(BlogEntrySchema, folderish=True, moveDiscussion=False)
BlogEntrySchema['effectiveDate'].widget.visible = {'view': 'invisible',
                                                    'edit': 'invisible'}
BlogEntrySchema['expirationDate'].widget.visible = {'view': 'invisible',
                                                    'edit': 'invisible'}

# #hide some fields
BlogEntrySchema['subject'].widget.visible = -1
BlogEntrySchema.changeSchemataForField('subject', 'default')
BlogEntrySchema['location'].widget.visible = -1
BlogEntrySchema.changeSchemataForField('location', 'default')
BlogEntrySchema['language'].widget.visible = -1
BlogEntrySchema.changeSchemataForField('language', 'default')
BlogEntrySchema['description'].widget.visible = -1


class BlogEntry(folder.ATFolder):
    """Ftw Blog Entry"""
    implements(IBlogEntry)

    portal_type = "BlogEntry"
    schema = BlogEntrySchema

    text = atapi.ATFieldProperty('text')

    #returns the category uid and the parent category uid
    def getCategoryUids(self):
        cats = aq_inner(self).getCategories()
        uids = [c.UID() for c in cats]
        parent_uids = []
        for pc in cats:
            parent = aq_inner(pc).aq_parent
            puid = parent.UID()
            grand_parent = aq_inner(parent).aq_parent
            if puid not in parent_uids \
                and grand_parent.Type() == 'Blog Category':
                parent_uids.append(puid)
                DateTime(self.CreationDate()).strftime('%m/%Y')
        return parent_uids + uids


registerType(BlogEntry, PROJECTNAME)



from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.blog.interfaces import IBlogEntryView
from ftw.tagging.interfaces.tagging import ITagRoot
from ftw.tagging.utils import getInterfaceRoot
from zope.interface import implements


class BlogEntryView(BrowserView):
    """ The Blog entry detail View. """

    implements(IBlogEntryView)
    template=ViewPageTemplateFile("entry.pt")

    def __call__(self):
        self.tag_root = getInterfaceRoot(self.context, ITagRoot)
        return self.template()



from Acquisition import aq_inner
from Products.CMFPlone.browser.interfaces import ISitemapView
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.blog.interfaces import IBlogUtils
from plone.app.portlets.portlets import base
from plone.portlets.interfaces import IPortletDataProvider
from zope.component import getMultiAdapter, getUtility
from zope.interface import implements


class ICategoriesPortlet(IPortletDataProvider):
    """
    """


class Assignment(base.Assignment):
    implements(ICategoriesPortlet)

    @property
    def title(self):
        return "Blog Categories Portlet"


class Renderer(base.Renderer):

    def __init__(self, *args, **kwargs):
        super(Renderer, self).__init__(*args, **kwargs)
        blogutils = getUtility(IBlogUtils, name='ftw.blog.utils')
        self.root = blogutils.getBlogRoot(self.context)

    @property
    def available(self):
        if not self.root:
            return False

        categories = getattr(self.root, 'categories', None)
        if not categories or not getattr(categories, 'objectIds', None):
            return False

        if not categories.objectIds():
            return False

        return True

    def update(self):
        if self.root:
            self.blogroot = self.root.absolute_url()
        else:
            self.blogroot = ''

    render = ViewPageTemplateFile('categories.pt')


class AddForm(base.NullAddForm):

    def create(self):
        return Assignment()


class CategoryPortletSitemapView(BrowserView):
    implements(ISitemapView)

    def createSiteMap(self):
        context = aq_inner(self.context)

        view = getMultiAdapter((context, self.request),
                               name='category_widget_builder_view')
        data = view.CategoryMap()

        #properties = getToolByName(context, 'portal_properties')
        #navtree_properties = getattr(properties, 'navtree_properties')
        #bottomLevel = navtree_properties.getProperty('bottomLevel', 0)
        # XXX: The recursion should probably be done in python code
        return context.category_portlet_recurs_view(
            children=data.get('children', []),
            level=0, bottomLevel=0)



from ftw.testing.layer import ComponentRegistryLayer


class ZCMLLayer(ComponentRegistryLayer):
    """A layer which only sets up the zcml, but does not start a zope
    instance.
    """

    def setUp(self):
        super(ZCMLLayer, self).setUp()
        import ftw.blog
        self.load_zcml_file('tests.zcml', ftw.blog.tests)
        self.load_zcml_file('configure.zcml', ftw.blog)


ZCML_LAYER = ZCMLLayer()



from Products.PloneTestCase import ptc
from collective.testcaselayer import common
from collective.testcaselayer import ptc as tcl_ptc


class Layer(tcl_ptc.BasePTCLayer):
    """Install ftw.blog """

    def afterSetUp(self):
        ptc.installPackage('ftw.blog')
        self.addProfile('ftw.blog:default')

layer = Layer([common.common_layer])



from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from ftw.blog.interfaces import IBlog
from ftw.blog.interfaces import IBlogUtils
from ftw.blog.testing import ZCML_LAYER
from ftw.blog.utility import BlogUtils
from ftw.testing import MockTestCase
from zope.component import getUtility
from zope.interface.verify import verifyClass


class TestUtility(MockTestCase):

    layer = ZCML_LAYER

    def test_component_registered(self):
        utils = getUtility(IBlogUtils, name='ftw.blog.utils')
        self.assertEqual(type(utils), BlogUtils)

    def test_component_implements_interface(self):
        self.assertTrue(IBlogUtils.implementedBy(BlogUtils))
        verifyClass(IBlogUtils, BlogUtils)

    def test_getBlogRoot_within_blog(self):
        blog = self.providing_stub([IBlog])
        obj = self.set_parent(
            self.stub(), self.set_parent(
                self.stub(), blog))

        self.replay()

        utils = getUtility(IBlogUtils, name='ftw.blog.utils')
        self.assertEqual(utils.getBlogRoot(obj), blog)

    def test_getBlogRoot_with_blog(self):
        blog = self.providing_stub([IBlog])

        self.replay()

        utils = getUtility(IBlogUtils, name='ftw.blog.utils')
        self.assertEqual(utils.getBlogRoot(blog), blog)

    def test_getBlogRoot_without_blog(self):
        site = self.providing_stub([IPloneSiteRoot])
        obj = self.set_parent(self.stub(), site)

        self.replay()

        utils = getUtility(IBlogUtils, name='ftw.blog.utils')
        self.assertEqual(utils.getBlogRoot(obj), None)



import os
from setuptools import setup, find_packages

version = '1.2'

maintainer = 'Mathias Leimgruber'

tests_require = [
    'collective.testcaselayer',
    'ftw.testing',
    'plone.testing',
    ]

extras_require = {
    'tests': tests_require,
    'tabbeview': [
        'ftw.tabbedview']}

setup(name='ftw.blog',
      version=version,
      description="A Blog for Plone",
      long_description=open("README.rst").read() + "\n" + \
          open(os.path.join("docs", "HISTORY.txt")).read(),

      # Get more strings from
      # http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Framework :: Plone :: 4.0",
        "Framework :: Plone :: 4.1",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],

      keywords='ftw blog',
      author='4teamwork GmbH',
      author_email='mailto:info@4teamwork.ch',
      maintainer=maintainer,
      url='https://github.com/4teamwork/ftw.blog',
      license='GPL2',

      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw'],
      include_package_data=True,
      zip_safe=False,

      install_requires=[
        'setuptools',
        'plone.browserlayer',
        'ftw.tagging',
        'Plone',
        # -*- Extra requirements: -*-
        ],
      tests_require=tests_require,
      extras_require=extras_require,

      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from plone.app.layout.viewlets import ViewletBase
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.tagging.browser.viewlets.taglist import TagListViewlet

# TODO : Evtl. it can be solved with a better solution

# Use a Browserpage instead of the viewlet, 
# becauase we insert the serachbox with js
# For mor information have a look to entry.pt or blog.pt and plone.app.jquerytools.browser.overlayhelper.js

class FtwBlogActionsBar(ViewletBase):
  """ A Viewlet wich show the blog specific searchfield """
  render = ViewPageTemplateFile('ftw_blog_actionsbar.pt')

class HiddenTagList(TagListViewlet):
    """ Hide the Taglist from ftw.tagging """

    def update(self):
        pass
        
    def index(self):
        return ''




from zope.interface import Interface


class IBlogLayer(Interface):
    """ blog specific request layer interface
    """


class IBlogEntry(Interface):
    """Ftw BlogEntry"""


class IBlogCategory(Interface):
    """Ftw BlogCategory"""


class IBlog(Interface):
    """Ftw Blog"""


class ICategoryWidget(Interface):
    """
    """


class IBlogView(Interface):
    """
    Marker Interface for BlogView
    """


class IBlogEntryView(Interface):
    """
    Marker Interface for BlogView
    """


class IBlogUtils(Interface):
    """
    provides Blog utilities
    """

    def getBlogRoot(context):
        """Returns the blog root of the current context. If the context is
        not within a blog, `None` is returned.
        """
